using System.ComponentModel;
using System.Xml.Serialization;

namespace EQIA.EQIDL
{
    /// <summary>
    /// Template used to draw the frame of a screen piece, stores a reference to animations used to draw the frame, and provides a draw function.
    /// </summary>
    [TypeConverter(typeof(FrameTemplateConverter))]
    public partial class FrameTemplate : Class
    {
        #region Members

        private string myTopLeft, myTop, myTopRight;
        private string myRightTop, myRight, myRightBottom;
        private string myBottomRight, myBottom, myBottomLeft;
        private string myLeftTop, myLeft, myLeftBottom;
        private string myMiddle;
        private int myOverlapLeft, myOverlapTop, myOverlapRight, myOverlapBottom;

        #endregion

        #region Properties

        /// <summary>
        /// Name of the animation to use for the top-left corner of this frame.
        /// </summary>
        [XmlElement("TopLeft", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the top-left corner of this frame.")]
        [NotifyParentProperty(true)]
        public string TopLeft { get { return myTopLeft; } set { myTopLeft = value; } }

        /// <summary>
        /// Name of the animation to use for the top edge of this frame.
        /// </summary>
        [XmlElement("Top", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the top edge of this frame.")]
        [NotifyParentProperty(true)]
        public string Top { get { return myTop; } set { myTop = value; } }

        /// <summary>
        /// Name of the animation to use for the top-right corner of this frame.
        /// </summary>
        [XmlElement("TopRight", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the top-right corner of this frame.")]
        [NotifyParentProperty(true)]
        public string TopRight { get { return myTopRight; } set { myTopRight = value; } }

        /// <summary>
        /// Name of the animation to use for the bottom-left corner of this frame.
        /// </summary>
        [XmlElement("BottomLeft", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the bottom-left corner of this frame.")]
        [NotifyParentProperty(true)]
        public string BottomLeft { get { return myBottomLeft; } set { myBottomLeft = value; } }

        /// <summary>
        /// Name of the animation to use for the bottom edge of this frame.
        /// </summary>
        [XmlElement("Bottom", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the bottom edge of this frame.")]
        [NotifyParentProperty(true)]
        public string Bottom { get { return myBottom; } set { myBottom = value; } }

        /// <summary>
        /// Name of the animation to use for the bottom-right corner of this frame.
        /// </summary>
        [XmlElement("BottomRight", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the bottom-right corner of this frame.")]
        [NotifyParentProperty(true)]
        public string BottomRight { get { return myBottomRight; } set { myBottomRight = value; } }

        /// <summary>
        /// Name of the animation to use for the left-top corner (below the top-left corner) of this frame.
        /// </summary>
        [XmlElement("LeftTop", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the left-top corner (below the top-left corner) of this frame.")]
        [NotifyParentProperty(true)]
        public string LeftTop { get { return myLeftTop; } set { myLeftTop = value; } }

        /// <summary>
        /// Name of the animation to use for the left edge of this frame.
        /// </summary>
        [XmlElement("Left", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the left edge of this frame.")]
        [NotifyParentProperty(true)]
        public string Left { get { return myLeft; } set { myLeft = value; } }

        /// <summary>
        /// Name of the animation to use for the left-bottom corner (above the bottom-left corner) of this frame.
        /// </summary>
        [XmlElement("LeftBottom", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the left-bottom corner (above the bottom-left corner) of this frame.")]
        [NotifyParentProperty(true)]
        public string LeftBottom { get { return myLeftBottom; } set { myLeftBottom = value; } }

        /// <summary>
        /// Name of the animation to use for the right-top corner (below the top-right corner) of this frame.
        /// </summary>
        [XmlElement("RightTop", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the right-top corner (below the top-right corner) of this frame.")]
        [NotifyParentProperty(true)]
        public string RightTop { get { return myRightTop; } set { myRightTop = value; } }

        /// <summary>
        /// Name of the animation to use for the right edge of this frame.
        /// </summary>
        [XmlElement("Right", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the right edge of this frame.")]
        [NotifyParentProperty(true)]
        public string Right { get { return myRight; } set { myRight = value; } }

        /// <summary>
        /// Name of the animation to use for the right-bottom corner (above the bottom-right corner) of this frame.
        /// </summary>
        [XmlElement("RightBottom", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the right-bottom corner (above the bottom-right corner) of this frame.")]
        [NotifyParentProperty(true)]
        public string RightBottom { get { return myRightBottom; } set { myRightBottom = value; } }

        /// <summary>
        /// Name of the animation to use for the middle of the frame.
        /// </summary>
        [XmlElement("Middle", IsNullable = false)]
        [Browsable(true), Description("Name of the animation to use for the middle of the frame.")]
        [NotifyParentProperty(true)]
        public string Middle { get { return myMiddle; } set { myMiddle = value; } }

        /// <summary>
        /// Amount of pixels to overlap the middle over the top edge.
        /// </summary>
        [XmlElement("OverlapTop", IsNullable = false)]
        [Browsable(true), Description("Amount of pixels to overlap the middle over the top edge.")]
        [NotifyParentProperty(true)]
        public int OverlapTop { get { return myOverlapTop; } set { myOverlapTop = value; } }

        /// <summary>
        /// Amount of pixels to overlap the middle over the bottom edge.
        /// </summary>
        [XmlElement("OverlapBottom", IsNullable = false)]
        [Browsable(true), Description("Amount of pixels to overlap the middle over the bottom edge.")]
        [NotifyParentProperty(true)]
        public int OverlapBottom { get { return myOverlapBottom; } set { myOverlapBottom = value; } }

        /// <summary>
        /// Amount of pixels to overlap the middle over the left edge.
        /// </summary>
        [XmlElement("OverlapLeft", IsNullable = false)]
        [Browsable(true), Description("Amount of pixels to overlap the middle over the left edge.")]
        [NotifyParentProperty(true)]
        public int OverlapLeft { get { return myOverlapLeft; } set { myOverlapLeft = value; } }

        /// <summary>
        /// Amount of pixels to overlap the middle over the right edge.
        /// </summary>
        [XmlElement("OverlapRight", IsNullable = false)]
        [Browsable(true), Description("Amount of pixels to overlap the middle over the right edge.")]
        [NotifyParentProperty(true)]
        public int OverlapRight { get { return myOverlapRight; } set { myOverlapRight = value; } }

        #endregion

        #region Creation

        public FrameTemplate()
            : this(null)
        {
        }

        public FrameTemplate(EQInterface eqInterface)
            : base(eqInterface)
        {
        }

        #endregion

        #region Deserialization

        protected override void SetProperties(object obj)
        {
            base.SetProperties(obj);

            FrameTemplate temp_FrameTemplate = obj as FrameTemplate;
            if (temp_FrameTemplate != null)
            {
                TopLeft = temp_FrameTemplate.TopLeft;
                Top = temp_FrameTemplate.Top;
                TopRight = temp_FrameTemplate.TopRight;
                RightTop = temp_FrameTemplate.RightTop;
                Right = temp_FrameTemplate.Right;
                RightBottom = temp_FrameTemplate.RightBottom;
                BottomRight = temp_FrameTemplate.BottomRight;
                Bottom = temp_FrameTemplate.Bottom;
                BottomLeft = temp_FrameTemplate.BottomLeft;
                LeftTop = temp_FrameTemplate.LeftTop;
                Left = temp_FrameTemplate.Left;
                LeftBottom = temp_FrameTemplate.LeftBottom;
                Middle = temp_FrameTemplate.Middle;
                OverlapLeft = temp_FrameTemplate.OverlapLeft;
                OverlapTop = temp_FrameTemplate.OverlapTop;
                OverlapRight = temp_FrameTemplate.OverlapRight;
                OverlapBottom = temp_FrameTemplate.OverlapBottom;
            }
        }

        #endregion
    }

    public class FrameTemplateConverter : ExpandableObjectConverter
    {
        public override object ConvertTo(ITypeDescriptorContext context, System.Globalization.CultureInfo culture, object value, System.Type destinationType)
        {
            return (destinationType == typeof(string)) ? string.Empty : base.ConvertTo(context, culture, value, destinationType);
        }
    }
}
